# Expose the following functions to the main package
from .cycling_query_new import cycling_query
from .escafish_score_new import escafish_score
from .exclude_region import exclude_region
from .generate_exclude import generate_exclude
from .get_oligos import get_oligos
from .HUSH_feedback import HUSH_feedback
from .reform_hush_combined_new import reform_hush_combined
from .reform_hush_consec import reform_hush_consec
from .reform_hush import reform_hush
from .select_probe import select_probe
from .split_fasta import split_fasta
from .summarize_probes_cumul import summarize_probes_cumul
from .summarize_probes_final import summarize_probes_final
from .summarize_probes import summarize_probes
from .download_chromosomes import download_chr_list, download_chr, download_ref_genome


__all__ = ["cycling_query_new",
            "escafish_score_new",
            "exclude_region",
            "generate_exclude",
            "get_oligos",
            "HUSH_feedback",
            "reform_hush_combined_new",
            "reform_hush_consec",
            "reform_hush",
            "select_probe",
            "split_fasta",
            "summarize_probes_cumul",
            "summarize_probes_final",
            "summarize_probes",
            "download_chr_list",
            "download_chr",
            "download_ref_genome"]

import os
# PATHMAIN is different from main init file
PATHMAIN = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
PATHSRC = os.path.join(PATHMAIN, "src")
PATHSHELL = os.path.join(PATHMAIN, "shell")
PATHDATA = os.path.join(PATHMAIN, "data")
PATHNOTEBOOK = os.path.join(PATHMAIN, "notebooks")

__constants__ = ["PATHMAIN",
                "PATHSRC",
                "PATHSHELL",
                "PATHDATA",
                "PATHNOTEBOOK"]

PATHS = [PATHMAIN, PATHSRC, PATHSHELL, PATHDATA, PATHNOTEBOOK]


from Bio.SeqIO.FastaIO import SimpleFastaParser
import numpy as np
import pandas as pd
import os
import glob
import sys
from tqdm import tqdm 
from tabulate import tabulate
import statistics as stat


def summarize_probes(currentfolder:os.PathLike = './data/')->None:
    # identify probe files
    pattern = currentfolder+"probe_candidates/**/probe_*.tsv"   #probelet
    #pattern = "data/**/probe*/**oligos.tsv"    #ifpd2 query
    #pattern = "data/**/*.best_probe.tsv"   #jupyternb
    filenames = glob.glob(pattern,recursive=True)

    roilist = currentfolder+'/rois/all_regions.tsv'
    rdroi = pd.read_csv(roilist,sep="\t",header=0)


    table = pd.DataFrame(columns=['folder', 'probe_set', 'roi', 'chr', 'start', 'end', \
            'nOligos', 'pw', 'size', 'coverage', \
            'centrality', \
            'd_mean', 'd_min', 'd_max', 'd_std', \
            'tm_range', 'tm_mean', 'tm_std', \
            'gc_range', 'gc_mean', 'gc_std', \
            'mean_closestMM', 'min_closestMM', 'max_closestMM', 'std_closestMM', \
            'mean_oligo_cost', 'min_oligo_cost', 'max_oligo_cost', 'std_oligo_cost' ])
            #'mean_sumMM', 'min_sumMM', 'max_sumMM', 'std_sumMM'   ])
        

    # read probe files
    for file in tqdm(filenames, "Processing probe candidates"):
        rd = pd.read_csv(file,sep="\t")
        last = len(table)+1
        
        # find coordinates of the corresponding roi
        current_roi = rd.name[1][4:]
        for k in range(len(rdroi)):
            if str(rdroi.window_id[k]) == current_roi:
                current = k

        roi_start = rdroi.Window_start[current]
        roi_end = rdroi.Window_end[current]
        roi_center = ((roi_start+roi_end)/2)-roi_start  #adjusted roi center

    # calculate summary measures and add as last row

    # replace off_target_no by min_mismatch for new queries
        probe_start = min(rd.start[1:])
        probe_end = max(rd.end[1:])
        probe_center = ((probe_start+probe_end)/2)-roi_start    #adjusted probe center

        filesplit = file.split("/")

        table.loc[last] = [filesplit[2], filesplit[3], rd.name[1], rd.chromosome[1], probe_start, probe_end, \
            len(rd), filesplit[2][-4:], probe_end-probe_start+1, (probe_end-probe_start+1)/(int(roi_end)-int(roi_start)), \
            min(probe_center/roi_center,2-(probe_center/roi_center)), \
            (rd.start-rd.end.shift()).mean(), (rd.start-rd.end.shift()).min(), (rd.start-rd.end.shift()).max(), (rd.start-rd.end.shift()).std(), \
            max(rd.Tm)-min(rd.Tm), stat.mean(rd.Tm), stat.stdev(rd.Tm), \
            max(rd.gc_content)-min(rd.gc_content), stat.mean(rd.gc_content), stat.stdev(rd.gc_content), \
            stat.mean(rd.off_target_no), min(rd.off_target_no), max(rd.off_target_no), stat.stdev(rd.off_target_no), \
            stat.mean(rd.oligo_cost), min(rd.oligo_cost), max(rd.oligo_cost), stat.stdev(rd.oligo_cost)]
        # stat.mean(rd.off_target_sum), min(rd.off_target_sum), max(rd.off_target_sum), stat.stdev(rd.off_target_sum)]  
    

    # append to existing probe summary file
    output = currentfolder+"probe_summary_probelet.tsv"
    sumExists = os.path.isfile(output)
    #if (sumExists):
    #    outtable = pd.read_csv(output,sep="\t")
    #    newtable = pd.concat([outtable, table])
    #else:
    newtable = table   

    newtable.sort_values(by=['probe_set'], inplace=True)

    # write into probe summary file
    newtable.to_csv(output,index=False,sep="\t")

    return

if __name__ == '__main__':
    summarize_probes()


#!/usr/bin/python3


# TO DO: 
# Switch to click arguments


from Bio.SeqIO.FastaIO import SimpleFastaParser
from ifpd2q.scripts.extract_kmers import main as extract
import pandas as pd
import os
import sys
from tqdm import tqdm 
from tabulate import tabulate

# Retrieve complete sequences in ROI
def get_oligos(nt_type:str='DNA',
               gcfilter:bool = 1, 
               extfolder:os.PathLike = './data/',
               reffile:os.PathLike|None= None,
               )->None:

    if nt_type=='DNA':
        roifile = os.path.join(extfolder,'rois/all_regions.tsv')
        ref = os.path.join(extfolder,'ref/')
        outseq = os.path.join(extfolder,'regions/')

        try:
            os.mkdir(outseq)
        except FileExistsError:
            print("Saving to existing 'regions' directory.")
            
        f = open(roifile)
        rd = pd.read_csv(f,sep="\t",header=0)

        for k in tqdm(range(len(rd)),desc='Retrieving sequences for all ROIs'):
            if not pd.isnull(rd.ref[k]):
                chrnr = rd.chrom[k][3:] # chromosome number

                if reffile is None: # if no reference file is specified, use the one in the rois file
                    reffile = os.path.join(ref,f"{rd.at[k,'ref']}.chromosome.{chrnr}.fa")

                with open(reffile) as handle:
                    for values in SimpleFastaParser(handle):
                        fullseq = values[1]
                        seq = fullseq[rd.Window_start[k]-1:rd.Window_end[k]]         # shift index by 1 to match ref genome

                        # export sequences
                        out = open(os.path.join(outseq,'roi_'+str(rd.window_id[k])+'.fa'),'w')
                        out.write('>ROI_'+str(rd.window_id[k])+' pos='+rd.chrom[k]+':'+str(rd.Window_start[k])+'-'+str(rd.Window_end[k])+'\n'+seq)
                        out.close()       
                        
                        
        # Divide into k-mers
        outcan = os.path.join(extfolder,'candidates/')

        try:
            os.mkdir(outcan)
        except FileExistsError:
            print("Saving to existing 'candidates' directory.")
            
        for k in range(len(rd)):
            fullseq = os.path.join(outseq,f'roi_{rd.window_id[k]}.fa')
            if not os.path.isfile(fullseq):
                print('The FASTA sequence for ROI '+str(rd.window_id[k])+' is missing.')
                continue
            extract(fullseq,outcan,rd.length[k],gcfilter)

    elif type == 'RNA':
        roifile = os.path.join(extfolder,'rois/all_regions.tsv')
        ref = os.path.join(extfolder,'ref/')
        outseq = os.path.join(extfolder,'regions/')

        # ASSUME THAT THE TRANSCRIPT SEQUENCES HAVE ALREADY BEEN IDENTIFIED
        # implement direct transcript retrieval? 

        f = open(roifile)
        rd = pd.read_csv(f,sep="\t",header=0)              
                        
        # Divide into k-mers
        outcan = os.path.join(extfolder,'candidates/')

        try:
            os.mkdir(outcan)
        except FileExistsError:
            print("Saving to existing 'candidates' directory.")
            
        for k in range(len(rd)):
            fullseq = os.path.join(outseq,f"roi_{rd.window_id[k]}.fa")
            if not os.path.isfile(fullseq):
                print(f'The FASTA sequence for ROI {rd.window_id[k]} is missing.')
                continue
            extract(fullseq,outcan,rd.length[k],gcfilter)
    else:
        print(f'Invalid nt_type: {nt_type}. Exiting...')
        exit(-1)

    return   

if __name__ == '__main__':
    #syntax: ./get_oligos.py DNA/RNA gcfilter extfolder

    if(len(sys.argv) == 1):
        # no probe type was specified
        get_oligos(nt_type='DNA') # default is nt_type = 'DNA'
    elif(len(sys.argv) == 2):
        get_oligos(nt_type = sys.argv[1]) # default is nt_type = 'DNA'
    elif(len(sys.argv) == 3):
        get_oligos(nt_type = sys.argv[1],
                   gcfilter= sys.argv[2]) # gcfilter = 1 (True) by default
    elif(len(sys.argv) == 4):
        get_oligos(nt_type = sys.argv[1],
                   gcfilter = sys.argv[2],
                   extfolder = sys.argv[3]) # extraction folder = './data/' by default

    else:
         print(f'Incorrect number of arguments. Exiting...')
         exit(-1)
    


# Expose this functions to outside as a module
from .src import (
    cycling_query,
    escafish_score,
    exclude_region,
    generate_exclude,
    get_oligos,
    HUSH_feedback,
    reform_hush_combined,
    reform_hush_consec,
    reform_hush,
    select_probe,
    split_fasta,
    summarize_probes_cumul,
    summarize_probes_final,
    summarize_probes,
    download_chr_list,
    download_chr,
    download_ref_genome)


__all__ = ["cycling_query",
            "escafish_score",
            "exclude_region",
            "generate_exclude",
            "get_oligos",
            "HUSH_feedback",
            "reform_hush_combined",
            "reform_hush_consec",
            "reform_hush",
            "select_probe",
            "split_fasta",
            "summarize_probes_cumul",
            "summarize_probes_final",
            "summarize_probes",
            "download_chr_list",
            "download_chr",
            "download_ref_genome",
            ]

# CONSTANTS
import os

PATHMAIN = os.path.dirname(os.path.abspath(__file__))
PATHSRC = os.path.join(PATHMAIN, "src")
PATHSHELL = os.path.join(PATHMAIN, "shell")
PATHDATA = os.path.join(PATHMAIN, "data")
PATHNOTEBOOK = os.path.join(PATHMAIN, "notebooks")

__constants__ = ["PATHMAIN",
                "PATHSRC",
                "PATHSHELL",
                "PATHDATA",
                "PATHNOTEBOOK"]

PATHS = [PATHMAIN, PATHSRC, PATHSHELL, PATHDATA, PATHNOTEBOOK]


import requests
from bs4 import BeautifulSoup



# get the website
ws = requests.get("https://www.ensembl.org/{Homo_sapiens}/Info/Index".format())

soup = BeautifulSoup(ws.content, 'html.parser')
title = soup.find('title').string

# get the species and build number
species = title.split()[0]
build = title.split()[-1]
